@@ -12,6 +12,7 @@
 # PURPOSE, OR NON-INFRINGEMENT.  MAJORMODE SHALL NOT BE LIABLE FOR ANY
 # LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 # OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
+
 import uuid
 
 from majormode.perseus.constant.http import HttpMethod
@@ -80,13 +81,20 @@ class DeviceServiceHttpRequestHandler(HttpRequestHandler):
                   authentication_required=False,
                   signature_required=True)
     def report_battery_events(self, request, device_id):
-        events = request.get_argument(
-            'events',
-            data_type=HttpRequest.ArgumentDataType.list,
-            item_data_type=HttpRequest.ArgumentDataType.object,
-            object_class=BatteryStateChangeEvent,
-            is_required=True
-        )
+        # events = request.get_argument(
+        #     'events',
+        #     data_type=HttpRequest.ArgumentDataType.list,
+        #     item_data_type=HttpRequest.ArgumentDataType.object,
+        #     object_class=BatteryStateChangeEvent,
+        #     is_required=True
+        # )
+        if not isinstance(request.body, list):
+            raise DeviceService.InvalidArgumentException()
+
+        events = [
+            BatteryStateChangeEvent.from_json(payload)
+            for payload in request.body
+        ]
 
         token = request.get_argument(
             'token',
@@ -155,13 +163,20 @@ class DeviceServiceHttpRequestHandler(HttpRequestHandler):
                   authentication_required=False,
                   signature_required=True)
     def report_location_events(self, request, device_id):
-        events = request.get_argument(
-            'events',
-            data_type=HttpRequest.ArgumentDataType.list,
-            item_data_type=HttpRequest.ArgumentDataType.object,
-            object_class=LocationUpdate,
-            is_required=True
-        )
+        # events = request.get_argument(
+        #     'events',
+        #     data_type=HttpRequest.ArgumentDataType.list,
+        #     item_data_type=HttpRequest.ArgumentDataType.object,
+        #     object_class=LocationUpdate,
+        #     is_required=True
+        # )
+        if not isinstance(request.body, list):
+            raise DeviceService.InvalidArgumentException()
+
+        events = [
+            LocationUpdate.from_json(payload)
+            for payload in request.body
+        ]
 
         token = request.get_argument(
             'token',
@@ -352,23 +367,7 @@ class DeviceServiceHttpRequestHandler(HttpRequestHandler):
     #     return XeberusService().on_device_alert_message(request.app_id, device_id, seed, state_changes)
     #
     #
-    #
-    #
 
-
-    #
-    #
-    #
-    #
-    #
-    #
-    #
-    #
-    #
-    #
-    #
-    #
-    #
     # @http_request(r'/device/(device_id)/allocation$',
     #               http_method=HttpRequest.HttpMethod.POST,
     #               authentication_required=True,
